"""
Version information for the Karo instrumentation.
"""
# This file exists primarily for consistency with other instrumentors,
# but the actual version is fetched in __init__.py.

# Define a dummy version here if __init__.py is the primary source
# The real values come from importlib in __init__.py
LIBRARY_NAME = "karo"
LIBRARY_VERSION = "0.4.1"