"""Attributes specific to instrumentation."""


class InstrumentationAttributes:
    """Instrumentation specific attributes."""

    NAME = "instrumentation.name"  # Name of the instrumentation
    VERSION = "instrumentation.version"  # Version of the instrumentation

    LIBRARY_NAME = "library.name"  # Name of the library
    LIBRARY_VERSION = "library.version"  # Version of the library

    INSTRUMENTATION_TYPE = "instrumentation.type"  # Type of instrumentation
    INSTRUMENTATION_PROVIDER = "instrumentation.provider"  # Provider of the instrumentation
